@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010, Albertas Vyšniauskas
+ * Copyright (c) 2009-2011, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -153,7 +153,7 @@ static void updateDiplays(ColorPickerArgs *args, GtkWidget *except_widget){
 	if (except_widget != args->cmyk_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->cmyk_control), &c);
 	if (except_widget != args->lab_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->lab_control), &c);
 
-	string color_name = color_names_get(args->gs->color_names, &c);
+	string color_name = color_names_get(args->gs->color_names, &c, true);
 	gtk_entry_set_text(GTK_ENTRY(args->color_name), color_name.c_str());
 
 	gtk_color_get_color(GTK_COLOR(args->contrastCheck), &c2);
@@ -211,7 +211,7 @@ static void on_swatch_menu_add_to_palette(GtkWidget *widget,  gpointer item) {
 	gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
 
 	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name=color_names_get(args->gs->color_names, &c);
+	string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
 	dynv_set_string(color_object->params, "name", name.c_str());
 	color_list_add_color_object(args->gs->colors, color_object, 1);
 	color_object_release(color_object);
@@ -224,7 +224,7 @@ static void on_swatch_menu_add_all_to_palette(GtkWidget *widget,  gpointer item)
 		gtk_swatch_get_color(GTK_SWATCH(args->swatch_display), i, &c);
 
 		struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-		string name=color_names_get(args->gs->color_names, &c);
+		string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
 		dynv_set_string(color_object->params, "name", name.c_str());
 		color_list_add_color_object(args->gs->colors, color_object, 1);
 		color_object_release(color_object);
@@ -237,7 +237,7 @@ static void on_swatch_color_activated(GtkWidget *widget, gpointer item) {
 	gtk_swatch_get_active_color(GTK_SWATCH(widget), &c);
 
 	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name=color_names_get(args->gs->color_names, &c);
+	string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
 	dynv_set_string(color_object->params, "name", name.c_str());
 	color_list_add_color_object(args->gs->colors, color_object, 1);
 	color_object_release(color_object);
@@ -277,7 +277,7 @@ static void paste_cb(GtkWidget *widget, ColorPickerArgs* args) {
 }
 
 static void swatch_popup_menu_cb(GtkWidget *widget, ColorPickerArgs* args){
- 	GtkWidget *menu;
+	GtkWidget *menu;
 
 	gint32 button, event_time;
 
@@ -285,12 +285,12 @@ static void swatch_popup_menu_cb(GtkWidget *widget, ColorPickerArgs* args){
 	updateMainColor(args);
 	gtk_swatch_get_main_color(GTK_SWATCH(args->swatch_display), &c);
 
-    struct ColorObject* color_object;
-    color_object = color_list_new_color_object(args->gs->colors, &c);
-    menu = converter_create_copy_menu(color_object, 0, args->gs);
+	struct ColorObject* color_object;
+	color_object = color_list_new_color_object(args->gs->colors, &c);
+	menu = converter_create_copy_menu(color_object, 0, args->gs);
 	color_object_release(color_object);
 
-    gtk_widget_show_all(GTK_WIDGET(menu));
+	gtk_widget_show_all(GTK_WIDGET(menu));
 
 	button = 0;
 	event_time = gtk_get_current_event_time ();
@@ -412,6 +412,42 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 			return FALSE;
 			break;
 
+		case GDK_1:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 1);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_2:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 2);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_3:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 3);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_4:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 4);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_5:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 5);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_6:
+			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 6);
+			updateDiplays(args, widget);
+			return TRUE;
+			break;
+
 		case GDK_Right:
 			gtk_swatch_move_active(GTK_SWATCH(args->swatch_display),1);
 			updateDiplays(args, widget);
@@ -435,7 +471,7 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 				gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
 
 				struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-				string name=color_names_get(args->gs->color_names, &c);
+				string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
 				dynv_set_string(color_object->params, "name", name.c_str());
 				color_list_add_color_object(args->gs->colors, color_object, 1);
 				color_object_release(color_object);
@@ -717,6 +753,7 @@ static int source_destroy(ColorPickerArgs *args){
 	dynv_set_int32(args->params, "sampler.falloff", sampler_get_falloff(args->gs->sampler));
 
 	dynv_set_float(args->params, "zoom", gtk_zoomed_get_zoom(GTK_ZOOMED(args->zoomed_display)));
+	dynv_set_int32(args->params, "zoom_size", gtk_zoomed_get_size(GTK_ZOOMED(args->zoomed_display)));
 
 	dynv_set_bool(args->params, "expander.settings", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderSettings)));
 	dynv_set_bool(args->params, "expander.rgb", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderRGB)));
@@ -743,7 +780,7 @@ static int source_get_color(ColorPickerArgs *args, ColorObject** color){
 	gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
 
 	struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
-	string name = color_names_get(args->gs->color_names, &c);
+	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
 	dynv_set_string(color_object->params, "name", name.c_str());
 
 	*color = color_object;
@@ -773,6 +810,8 @@ static int source_activate(ColorPickerArgs *args){
 	float refresh_rate = dynv_get_float_wd(args->global_params, "refresh_rate", 30);
 	args->timeout_source_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 1000/refresh_rate, (GSourceFunc)updateMainColorTimer, args, (GDestroyNotify)NULL);
 
+	gtk_zoomed_set_size(GTK_ZOOMED(args->zoomed_display), dynv_get_int32_wd(args->params, "zoom_size", 150));
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "focus_swatch"), "Click on swatch area to begin adding colors to palette");
 
 	return 0;
@@ -792,10 +831,8 @@ static int source_deactivate(ColorPickerArgs *args){
 
 static struct ColorObject* get_color_object(struct DragDrop* dd){
 	ColorPickerArgs* args=(ColorPickerArgs*)dd->userdata;
-	Color c;
-	gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
-	struct ColorObject* colorobject = color_object_new(dd->handler_map);
-	color_object_set_color(colorobject, &c);
+	ColorObject *colorobject;
+	source_get_color(args, &colorobject);
 	return colorobject;
 }
 
@@ -822,10 +859,15 @@ void color_picker_set_floating_picker(ColorSource *color_source, FloatingPicker
 }
 
 static struct ColorObject* get_color_object_contrast(struct DragDrop* dd){
+	ColorPickerArgs* args = static_cast<ColorPickerArgs*>(dd->userdata);
 	Color c;
 	gtk_color_get_color(GTK_COLOR(dd->widget), &c);
 	struct ColorObject* colorobject = color_object_new(dd->handler_map);
 	color_object_set_color(colorobject, &c);
+
+	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
+	dynv_set_string(colorobject->params, "name", name.c_str());
+
 	return colorobject;
 }
 
@@ -919,6 +961,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 
 			args->zoomed_display = gtk_zoomed_new();
+			gtk_zoomed_set_size(GTK_ZOOMED(args->zoomed_display), dynv_get_int32_wd(args->params, "zoom_size", 150));
 			gtk_box_pack_start (GTK_BOX(vbox), args->zoomed_display, false, false, 0);
 
 
@@ -960,7 +1003,6 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 				gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 				table_y++;
 
-
 			expander=gtk_expander_new("HSV");
 			gtk_expander_set_expanded(GTK_EXPANDER(expander), dynv_get_bool_wd(args->params, "expander.hsv", false));
 			args->expanderHSV=expander;
